@@ -8,14 +8,23 @@ from devpi_server.types import canraise, cached_property
 import devpi_server
 log = getLogger(__name__)
 
+def get_default_serverdir():
+    return os.environ.get("DEVPI_SERVERDIR", "~/.devpi/server")
+
 def addoptions(parser):
-    group = parser.addgroup("main", "main options")
+    group = parser.addgroup("main options")
     group.addoption("--version", action="store_true",
             help="show devpi_version (%s)" % devpi_server.__version__)
 
-    opt = group.addoption("--datadir", type=str, metavar="DIR",
-            default="~/.devpi/server",
-            help="directory for server data")
+    opt = group.addoption("--serverdir", type=str, metavar="DIR",
+            default=None,
+            help="directory for server data.  By default, "
+                 "$DEVPI_SERVERDIR is used if it exists, "
+                 "otherwise the default is '~/.devpi/server'")
+
+    group.addoption("--host",  type=str,
+            default="localhost",
+            help="domain/ip address to listen on")
 
     group.addoption("--port",  type=int,
             default=3141,
@@ -23,34 +32,63 @@ def addoptions(parser):
                  "--gendeploy, port+1 will be used to prevent "
                  "accidental clashes with ad-hoc runs.")
 
-    group.addoption("--host",  type=str,
-            default="localhost",
-            help="domain/ip address to listen on")
+    group.addoption("--outside-url",  type=str, dest="outside_url",
+            metavar="URL",
+            default=None,
+            help="the outside URL where this server will be reachable. "
+                 "Set this if you proxy devpi-server through a web server "
+                 "and the web server does not set or you want to override "
+                 "the custom X-outside-url header.")
 
     group.addoption("--refresh", type=float, metavar="SECS",
             default=60,
             help="interval for consulting changelog api of pypi.python.org")
 
-    group = parser.addgroup("deploy", "deployment options")
+    group.addoption("--bypass-cdn", action="store_true",
+            help="set this if you want to bypass pypi's CDN for access to "
+                 "simple pages and packages, in order to rule out cache-"
+                 "invalidation issues.  This will only work if you "
+                 "are not using a http proxy.")
+
+    group.addoption("--passwd", action="store", metavar="USER",
+            help="set password for user USER (interactive)")
+
+
     group.addoption("--gendeploy", action="store", metavar="DIR",
-            help="(unix only) generate a pre-configured self-contained "
+            help="(unix only) install and generate a pre-configured "
                  "virtualenv directory which puts devpi-server "
-                 "under supervisor control.  Also provides "
-                 "nginx/cron files to help with permanent deployment. ")
+                 "under supervisor control and provides some example "
+                 "files for nginx/crontab.  If the specified DIR exists "
+                 "virtualenv creation will be skipped and only the "
+                 "installation steps will be performed. ")
 
     group.addoption("--secretfile", type=str, metavar="path",
-            default="~/.devpi/server/.secret",
+            default="{serverdir}/.secret",
             help="file containing the server side secret used for user "
                  "validation. If it does not exist, a random secret "
                  "is generated on start up and used subsequently. ")
 
     group.addoption("--bottleserver", metavar="TYPE",
-            default="wsgiref",
-            help="bottle server class, you may try eventlet or others")
+            default="auto",
+            help="bottle server class, tries 'eventlet', "
+                 "then 'wsgiref'  if set to 'auto' (default)")
 
     group.addoption("--debug", action="store_true",
             help="run wsgi application with debug logging")
 
+    group = parser.addgroup("background server")
+    group.addoption("--start", action="store_true",
+            help="start the background devpi-server")
+    group.addoption("--stop", action="store_true",
+            help="stop the background devpi-server")
+    group.addoption("--status", action="store_true",
+            help="show status of background devpi-server")
+    group.addoption("--log", action="store_true",
+            help="show logfile content of background server")
+
+    group = parser.addgroup("deployment options")
+
+
 def try_argcomplete(parser):
     try:
         import argcomplete
@@ -60,9 +98,6 @@ def try_argcomplete(parser):
         argcomplete.autocomplete(parser)
 
 def parseoptions(argv, addoptions=addoptions):
-    if argv is None:
-        argv = sys.argv
-    argv = map(str, argv)
     parser = MyArgumentParser(
         description="Start an index server acting as a cache for "
                     "pypi.python.org, suitable for pip/easy_install usage. "
@@ -71,6 +106,7 @@ def parseoptions(argv, addoptions=addoptions):
     addoptions(parser)
     try_argcomplete(parser)
     args = parser.parse_args(argv[1:])
+    args._raw = argv[1:]
     config = Config(args)
     return config
 
@@ -112,7 +148,16 @@ class ConfigurationError(Exception):
 class Config:
     def __init__(self, args):
         self.args = args
-        self.secretfile = py.path.local(os.path.expanduser(args.secretfile))
+        serverdir = args.serverdir
+        if serverdir is None:
+            serverdir = get_default_serverdir()
+        self.serverdir = py.path.local(os.path.expanduser(serverdir))
+
+        if args.secretfile == "{serverdir}/.secret":
+            self.secretfile = self.serverdir.join(".secret", abs=True)
+        else:
+            self.secretfile = py.path.local(
+                    os.path.expanduser(args.secretfile))
 
     @cached_property
     def secret(self):
@@ -135,6 +180,14 @@ def getpath(path):
     return py.path.local(os.path.expanduser(str(path)))
 
 def render(tw, basedir, confname, format=None, **kw):
+    result = render_string(confname, format=format, **kw)
+    conf = basedir.join(confname)
+    conf.write(result)
+    if tw is not None:
+        tw.line("wrote %s" % conf, bold=True)
+    return conf
+
+def render_string(confname, format=None, **kw):
     template = confname + ".template"
     from pkg_resources import resource_string
     templatestring = resource_string("devpi_server.cfg", template)
@@ -144,8 +197,4 @@ def render(tw, basedir, confname, format=None, **kw):
         result = templatestring.format(**kw)
     else:
         result = templatestring % kw
-    conf = basedir.join(confname)
-    conf.write(result)
-    if tw is not None:
-        tw.line("wrote %s" % conf, bold=True)
-    return conf
+    return result
