@@ -16,31 +16,58 @@ def getpwhash(password, salt):
     hash.update(password)
     return hash.hexdigest()
 
-_ixconfigattr = set("type volatile bases".split())
+def run_passwd(db, user):
+    if not db.user_exists(user):
+        log.error("user %r not found" % user)
+        return 1
+    for i in range(3):
+        pwd = py.std.getpass.getpass("enter password for %s: " % user)
+        pwd2 = py.std.getpass.getpass("repeat password for %s: " % user)
+        if pwd != pwd2:
+            log.error("password don't match")
+        else:
+            break
+    else:
+        log.error("no password set")
+        return 1
+    db.user_modify(user, password=pwd)
+
+
+_ixconfigattr = set(
+    "type volatile bases uploadtrigger_jenkins acl_upload".split())
 
 class DB:
+    class InvalidIndexconfig(Exception):
+        def __init__(self, messages):
+            self.messages = messages
+            Exception.__init__(self, messages)
 
     def __init__(self, xom):
         self.xom = xom
         self.keyfs = xom.keyfs
 
-    # user handling
-    def user_setpassword(self, user, password):
+    def user_create(self, user, password, email=None):
         with self.keyfs.USER(user=user).update() as userconfig:
-            return self._setpassword(userconfig, user, password)
-
-    def user_create(self, user, password, email):
-        with self.keyfs.USER(user=user).update() as userconfig:
-            hash = self._setpassword(userconfig, user, password)
-            userconfig["email"] = email
+            self._setpassword(userconfig, user, password)
+            if email:
+                userconfig["email"] = email
             log.info("created user %r with email %r" %(user, email))
-            return hash
+
+    def user_modify(self, user, password=None, email=None):
+        with self.keyfs.USER(user=user).update() as userconfig:
+            modified = []
+            if password is not None:
+                self._setpassword(userconfig, user, password)
+                modified.append("password=*******")
+            if email:
+                userconfig["email"] = email
+                modified.append("email=%s" % email)
+            log.info("modified user %r: %s" %(user, ", ".join(modified)))
 
     def _setpassword(self, userconfig, user, password):
         userconfig["pwsalt"] = salt = os.urandom(16).encode("base_64")
         userconfig["pwhash"] = hash = getpwhash(password, salt)
         log.info("setting password for user %r", user)
-        return hash
 
     def user_delete(self, user):
         self.keyfs.USER(user=user).delete()
@@ -69,38 +96,99 @@ class DB:
             d["username"] = user
         return d
 
-    def user_indexconfig_get(self, user, index):
+    def _get_user_and_index(self, user, index=None):
+        if index is None:
+            user = user.strip("/")
+            user, index = user.split("/")
+        return user, index
+
+    def index_get(self, user, index=None):
+        user, index = self._get_user_and_index(user, index)
         userconfig = self.keyfs.USER(user=user).get()
         try:
-            return userconfig["indexes"][index]
+            indexconfig = userconfig["indexes"][index]
         except KeyError:
             return None
+        if "acl_upload" not in indexconfig:
+            indexconfig["acl_upload"] = [user]
+        return indexconfig
 
-    def user_indexconfig_set(self, user, index=None, **kw):
-        if index is None:
-            user, index = user.split("/")
-        assert kw["type"] in ("stage", "mirror")
+    def _normalize_bases(self, bases):
+        # check and normalize base indices
+        messages = []
+        newbases = []
+        for base in bases:
+            try:
+                base_user, base_index = self._get_user_and_index(base)
+            except ValueError:
+                messages.append("invalid base index spec: %r" % (base,))
+            else:
+                if self.index_get(base) is None:
+                    messages.append("base index %r does not exist" %(base,))
+                else:
+                    newbases.append("%s/%s" % (base_user, base_index))
+        if messages:
+            raise self.InvalidIndexconfig(messages)
+        return newbases
+
+    def index_get(self, user, index=None):
+        user, index = self._get_user_and_index(user, index)
+        userconfig = self.keyfs.USER(user=user).get()
+        return userconfig.get("indexes", {}).get(index)
+
+    def index_exists(self, user, index=None):
+        return bool(self.index_get(user, index))
+
+    def index_create(self, user, index=None, type="stage",
+                     volatile=True, bases=("root/pypi",),
+                     uploadtrigger_jenkins=None,
+                     acl_upload=None):
+        user, index = self._get_user_and_index(user, index)
+
+        if acl_upload is None:
+            acl_upload = [user]
+        bases = tuple(self._normalize_bases(bases))
+
+        # modify user/indexconfig
         with self.keyfs.USER(user=user).locked_update() as userconfig:
             indexes = userconfig.setdefault("indexes", {})
-            ixconfig = indexes.setdefault(index, {})
-            ixconfig.update(kw)
-            if not set(ixconfig) == _ixconfigattr:
-                raise ValueError("incomplete config: %s" % ixconfig)
-            log.debug("configure_index %s/%s: %s", user, index, ixconfig)
+            assert index not in indexes, indexes[index]
+            indexes[index] = ixconfig = dict(
+                type=type, volatile=volatile, bases=bases,
+                uploadtrigger_jenkins=uploadtrigger_jenkins,
+                acl_upload=acl_upload)
+            log.info("created index %s/%s: %s", user, index, ixconfig)
             return ixconfig
 
-    def user_indexconfig_delete(self, user, index):
+    def index_modify(self, user, index=None, **kw):
+        user, index = self._get_user_and_index(user, index)
+        diff = list(set(kw).difference(_ixconfigattr))
+        if diff:
+            raise self.InvalidIndexconfig(
+                ["invalid keys for index configuration: %s" %(diff,)])
+        if "bases" in kw:
+            kw["bases"] = tuple(self._normalize_bases(kw["bases"]))
+
+        # modify user/indexconfig
         with self.keyfs.USER(user=user).locked_update() as userconfig:
-            indexes = userconfig.get("indexes") or {}
+            ixconfig = userconfig["indexes"][index]
+            ixconfig.update(kw)
+            log.info("modified index %s/%s: %s", user, index, ixconfig)
+            return ixconfig
+
+    def index_delete(self, user, index=None):
+        user, index = self._get_user_and_index(user, index)
+        with self.keyfs.USER(user=user).locked_update() as userconfig:
+            indexes = userconfig.get("indexes", {})
             if index not in indexes:
                 log.info("index %s/%s not exists", user, index)
                 return False
             del indexes[index]
-
+            self._remove_indexdir(user, index)
             log.info("deleted index config %s/%s" %(user, index))
             return True
 
-    def delete_index(self, user, index):
+    def _remove_indexdir(self, user, index):
         p = self.keyfs.INDEXDIR(user=user, index=index).filepath
         if p.check():
             p.remove()
@@ -110,9 +198,8 @@ class DB:
     # stage handling
 
     def getstage(self, user, index=None):
-        if index is None:
-            user, index = user.split("/")
-        ixconfig = self.user_indexconfig_get(user, index)
+        user, index = self._get_user_and_index(user, index)
+        ixconfig = self.index_get(user, index)
         if not ixconfig:
             return None
         if ixconfig["type"] == "stage":
@@ -125,15 +212,8 @@ class DB:
     def getstagename(self, user, index):
         return "%s/%s" % (user, index)
 
-    def getindexconfig(self, stagename):
-        user, index = stagename.split("/")
-        return self.user_indexconfig_get(user, index)
-
-    def create_stage(self, user, index=None,
-                     type="stage", bases=("/root/pypi",),
-                     volatile=True):
-        self.user_indexconfig_set(user, index, type=type, bases=bases,
-                                  volatile=volatile)
+    def create_stage(self, user, index=None, **kw):
+        self.index_create(user, index, **kw)
         return self.getstage(user, index)
 
 
@@ -152,29 +232,47 @@ class PrivateStage:
         self.name = user + "/" + index
         self.ixconfig = ixconfig
 
-    def configure(self, **kw):
-        assert _ixconfigattr.issuperset(kw)
-        config = self.ixconfig
-        config.update(kw)
-        self.db.user_indexconfig_set(self.user, self.index, **config)
+    def can_upload(self, username):
+        return username in self.ixconfig.get("acl_upload", [])
+
+    def _reconfigure(self, **kw):
+        self.ixconfig = self.db.index_modify(self.name, **kw)
+
+    def _get_sro(self):
+        """ return stage resolution order. """
+        todo = [self]
+        seen = set()
+        while todo:
+            stage = todo.pop(0)
+            yield stage
+            seen.add(stage.name)
+            for base in stage.ixconfig["bases"]:
+                if base not in seen:
+                    todo.append(self.db.getstage(base))
 
     def op_with_bases(self, opname, **kw):
-        op_perstage = getattr(self, opname + "_perstage")
-        results = [(self, op_perstage(**kw))]
-        for base in self.ixconfig["bases"]:
-            stage = self.db.getstage(base)
-            base_entries = getattr(stage, opname)(**kw)
-            results.append((stage, base_entries))
+        opname += "_perstage"
+        results = []
+        for stage in self._get_sro():
+            stage_result = getattr(stage, opname)(**kw)
+            results.append((stage, stage_result))
         return results
 
     #
     # registering project and version metadata
     #
+    #class MetadataExists(Exception):
+    #    """ metadata exists on a given non-volatile index. """
+
     def register_metadata(self, metadata):
         name = metadata["name"]
         version = metadata["version"]
         key = self.keyfs.PROJCONFIG(user=self.user, index=self.index, name=name)
         with key.locked_update() as projectconfig:
+            #if not self.ixconfig["volatile"] and projectconfig:
+            #    raise self.MetadataExists(
+            #        "%s-%s exists on non-volatile %s" %(
+            #        name, version, self.name))
             versionconfig = projectconfig.setdefault(version, {})
             versionconfig.update(metadata)
         desc = metadata.get("description")
@@ -236,7 +334,12 @@ class PrivateStage:
                 if res == 404:
                     continue
                 return res
-            all_projectconfig.update(res)
+            for ver in res:
+                if ver not in all_projectconfig:
+                    all_projectconfig[ver] = res[ver]
+                else:
+                    l = all_projectconfig[ver].setdefault("+shadowing", [])
+                    l.append(res[ver])
         return all_projectconfig
 
     #
@@ -245,13 +348,21 @@ class PrivateStage:
 
     def getreleaselinks(self, projectname):
         all_links = []
+        basenames = set()
         for stage, res in self.op_with_bases("getreleaselinks",
                                           projectname=projectname):
             if isinstance(res, int):
                 if res == 404:
                     continue
                 return res
-            all_links.extend(res)
+            for entry in res:
+                if entry.eggfragment:
+                    key = entry.eggfragment
+                else:
+                    key = entry.basename
+                if key not in basenames:
+                    basenames.add(key)
+                    all_links.append(entry)
         all_links = sorted_by_version(all_links, attr="basename")
         all_links.reverse()
         return all_links
@@ -276,8 +387,8 @@ class PrivateStage:
         return sorted(all_names)
 
     def getprojectnames_perstage(self):
-        return sorted(self.keyfs.PROJCONFIG.listnames("name",
-                      user=self.user, index=self.index))
+        return self.keyfs.PROJCONFIG.listnames("name",
+                    user=self.user, index=self.index)
 
     def store_releasefile(self, filename, content):
         name, version = DistURL(filename).pkgname_and_version
