@@ -15,20 +15,22 @@ from .vendor._pip import HTMLPage
 
 from .types import propmapping
 from .urlutil import DistURL, joinpath
+from pkg_resources import Requirement
 
 from logging import getLogger
 assert __name__ == "devpi_server.extpypi"
 log = getLogger(__name__)
 
 CONCUCCRENT_CRAWL = False
-ALLOWED_ARCHIVE_EXTS = ".egg .tar.gz .tar.bz2 .tar .tgz .zip".split()
+ALLOWED_ARCHIVE_EXTS = ".egg .whl .tar.gz .tar.bz2 .tar .tgz .zip".split()
 ARCHIVE_SCHEMES = ("http", "https")
 
 
 class IndexParser:
 
     def __init__(self, projectname):
-        self.projectname = projectname.lower()
+        self.projectname_raw = projectname
+        self.projectname = Requirement.parse(projectname).project_name.lower()
         self.basename2link = {}
         self.crawllinks = set()
         self.egglinks = []
@@ -79,17 +81,20 @@ class IndexParser:
 
 def is_archive_of_project(newurl, targetname):
     if newurl._parsed.scheme not in ARCHIVE_SCHEMES:
-        log.warn("url has wrong scheme %r", newurl)
+        log.warn("url has unsupported scheme %r", newurl)
         return False
-    targetname = targetname.lower()
     nameversion, ext = newurl.splitext_archive()
     parts = re.split(r'-\d+', nameversion)
     projectname = parts[0]
+    if not projectname:
+        return False
+    try:
+        pname = Requirement.parse(projectname).project_name.lower()
+    except ValueError:
+        return False
     #log.debug("checking %s, projectname %r, nameversion %s", newurl, self.projectname, nameversion)
-    if len(parts) > 1 and ext.lower() in ALLOWED_ARCHIVE_EXTS and \
-       projectname.lower() == targetname:
-        return True
-    return False
+    return (len(parts) > 1 and ext.lower() in ALLOWED_ARCHIVE_EXTS and
+            pname == targetname)
 
 def parse_index(disturl, html, scrape=True):
     if not isinstance(disturl, DistURL):
@@ -157,6 +162,7 @@ def perform_crawling(extdb, result, numthreads=10):
 
 class ExtDB:
     name = "root/pypi"
+    ixconfig = dict(bases=(), volatile=False)
 
     def __init__(self, xom):
         self.keyfs = xom.keyfs
@@ -169,7 +175,10 @@ class ExtDB:
 
     def getprojectnames(self):
         """ return list of all projects which have been served. """
-        # XXX return full upstream list?
+        log.info("slow access (big reply) to full extpypi simple list")
+        if hasattr(self, "name2serials"):
+            return sorted(x for x in self.name2serials if x[0] != ".")
+        log.warn("pypi.python.org's simple page not cached yet")
         return sorted(self.keyfs.PYPILINKS.listnames("name"))
 
     getprojectnames_perstage = getprojectnames
@@ -202,6 +211,9 @@ class ExtDB:
         if res is not None and serial >= refresh:
             return [self.releasefilestore.getentry(relpath) for relpath in res]
 
+        # XXX short cut 404 if self.name2serials is not empty and
+        # does not contain the projectname (watch out case-sensitivity)
+
         url = PYPIURL_SIMPLE + projectname + "/"
         log.debug("visiting index %s", url)
         response = self.httpget(url, allow_redirects=True)
@@ -224,6 +236,11 @@ class ExtDB:
         self._dump_projectlinks(projectname, dumplist, serial)
         return entries
 
+    getreleaselinks_perstage = getreleaselinks
+
+    def op_with_bases(self, opname, **kw):
+        return [(self, getattr(self, opname)(**kw))]
+
     def get_projectconfig(self, name):
         releaselinks = self.getreleaselinks(name)
         if isinstance(releaselinks, int):
@@ -240,25 +257,37 @@ class ExtDB:
             files[url.basename] = link.relpath
         return data
 
+    get_projectconfig_perstage = get_projectconfig
+
     def get_description(self, name, version):
         link = "https://pypi.python.org/pypi/%s/%s/" % (name, version)
         return html.div("please refer to description on remote server ",
             html.a(link, href=link)).unicode(indent=2)
 
+    MIRRORVERSION = 1
     def spawned_pypichanges(self, proxy, proxysleep):
         log.info("changelog/update tasks starting")
         keyfs = self.keyfs
         name2serials = keyfs.PYPISERIALS.get({})
+        name2serials_version = name2serials.get(".ver", 0)
+        if name2serials_version != self.MIRRORVERSION:
+            log.info("detected MIRRORVERSION CHANGE, restarting caching info")
+            name2serials.clear() # restart getting info from pypi
+                                 # this will not reget release files
+        if name2serials:
+            self.name2serials = name2serials
         while 1:
             if not name2serials:
-                log.debug("retrieving initial name/serial list")
+                log.info("retrieving initial name/serial list")
                 init_name2serials = proxy.list_packages_with_serial()
                 if init_name2serials is None:
                     proxysleep()
                     continue
                 for name, serial in iteritems(init_name2serials):
                     name2serials[name.lower()] = serial
+                name2serials[".ver"] = self.MIRRORVERSION
                 keyfs.PYPISERIALS.set(name2serials)
+                self.name2serials = name2serials
             else:
                 # get changes since the maximum serial we are aware of
                 current_serial = max(itervalues(name2serials))
@@ -281,7 +310,7 @@ class ExtDB:
             # walk through all mirrored projects and trigger updates if needed
             for name in self.getcontained():
                 name = name.lower()
-                serial = name2serials[name]
+                serial = name2serials.get(name, 0)
                 self.getreleaselinks(name, refresh=serial)
             proxysleep()
 
