@@ -6,6 +6,7 @@ for all indexes.
 import hashlib
 import posixpath
 import os
+import json
 from wsgiref.handlers import format_date_time
 from datetime import datetime
 from time import mktime
@@ -25,10 +26,14 @@ class ReleaseFileStore:
         key = self.keyfs.PYPIFILES(relpath=link.torelpath())
         entry = self.getentry(key.relpath)
         mapping = {"url": link.geturl_nofragment().url}
-        if link.eggfragment and not entry.eggfragment:
-            mapping["eggfragment"] = link.eggfragment
-        elif link.md5 and not entry.md5:
-            mapping["md5"] = link.md5
+        mapping["eggfragment"] = link.eggfragment
+        mapping["md5"] = link.md5
+        if link.md5 != entry.md5:
+            if entry.FILE.exists():
+                log.info("replaced md5, deleting stale %s" % entry.relpath)
+                entry.FILE.delete()
+            else:
+                log.info("replaced md5 info for %s" % entry.relpath)
         entry.set(**mapping)
         assert entry.url
         return entry
@@ -120,6 +125,29 @@ class ReleaseFileStore:
                   last_modified=http_date())
         return entry
 
+    def add_attachment(self, md5, type, data):
+        assert type in ("toxresult",)
+        assert isinstance(data, bytes)
+        # XXX thread safety
+        num = len(self.keyfs.ATTACHMENT.listnames("num",
+                                                  type=type,
+                                                  md5=md5))
+        num = str(num)
+        key = self.keyfs.ATTACHMENT(type=type, md5=md5, num=num)
+        key.set(data)
+        return num
+
+    def get_attachment(self, md5, type, num):
+        return self.keyfs.ATTACHMENT(type=type, md5=md5, num=num).get()
+
+    def iter_attachments(self, md5, type):
+        import json
+        nums = self.keyfs.ATTACHMENT.listnames("num", type=type, md5=md5)
+        for num in range(len(nums)):
+            a = self.keyfs.ATTACHMENT(num=str(num), type=type, md5=md5).get()
+            yield json.loads(a)
+
+
 def getmd5(content):
     md5 = hashlib.md5()
     md5.update(content)
@@ -152,6 +180,9 @@ class RelPathEntry(object):
                  size = headers.get("content-length"),
                  last_modified = headers.get("last-modified"))
 
+    def exists(self):
+        return bool(self._mapping)
+
     def invalidate_cache(self):
         try:
             del self._mapping["_headers"]
@@ -184,6 +215,7 @@ class RelPathEntry(object):
             if val is not None:
                 mapping[name] = str(val)
         self._mapping.update(mapping)
+
         self.PATHENTRY.set(self._mapping)
 
 for _ in RelPathEntry._attr:
