@@ -3,6 +3,7 @@ import py
 import sys
 import os
 import subprocess
+from devpi_server import urlutil
 
 from devpi_server.config import render, getpath
 import devpi_server
@@ -15,54 +16,67 @@ def gendeploycfg(config, venvdir, tw=None):
         tw = py.io.TerminalWriter()
         tw.cwd = py.path.local()
 
-    tw.line("creating etc/ directory for supervisor configuration", bold=True)
+    #tw.line("creating etc/ directory for supervisor configuration", bold=True)
     etc = venvdir.ensure("etc", dir=1)
-    httpport = config.args.port + 1
-    datadir = venvdir.ensure("data", dir=1)
+    orig_args = list(config.args._raw)
+
+    # filter out --gendeploy option
+    for i, val in enumerate(orig_args):
+        if val.startswith("--gendeploy="):
+            del orig_args[i]
+            break
+        elif val == "--gendeploy":
+            del orig_args[i:i+2]
+            break
+
+    if not config.args.serverdir:  # default
+        serverdir = venvdir.ensure("data", dir=1)
+    else:
+        serverdir = config.serverdir
+    orig_args.extend(["--serverdir", str(serverdir)])
+
     logdir = venvdir.ensure("log", dir=1)
 
     render(tw, etc, "supervisord.conf", venvdir=venvdir,
-           devpiport=httpport,
-           logdir=logdir, datadir=datadir)
-    nginxconf = render(tw, etc, "nginx-devpi.conf", format=1, port=httpport,
-                       datadir=datadir)
-    indexurl = "http://localhost:%s/root/dev/+simple/" % httpport
-    devpictl = create_devpictl(tw, venvdir, httpport)
+           server_args=subprocess.list2cmdline(orig_args),
+           logdir=logdir)
+    outside_url = config.args.outside_url
+    if outside_url is None: # default
+        outside_host = "localhost"
+        outside_port = 80
+    else:
+        parsed = urlutil.urlparse(outside_url)
+        parts = list(parsed.netloc.split(":"))
+        if len(parts) < 2:
+            parts.append(80)
+        outside_host, outside_port = parts
+
+    nginxconf = render(tw, etc, "nginx-devpi.conf", format=1,
+                       outside_host=outside_host,
+                       outside_port=outside_port,
+                       port=config.args.port,
+                       serverdir=serverdir)
+    devpictl = create_devpictl(tw, venvdir)
     cron = create_crontab(tw, etc, devpictl)
     tw.line("created and configured %s" % venvdir, bold=True)
     tw.line(py.std.textwrap.dedent("""\
-    You may now execute the following:
+    To control supervisor's deployment of devpi-server set:
 
-         alias devpi-ctl='%(devpictl)s'
+        alias devpi-ctl='%(devpictl)s'
 
-    and then call:
+    and then start the server process:
 
         devpi-ctl start all
-
-    after which you can configure pip to always use the default
-    root/dev index which carries all pypi packages and the ones
-    you upload to it::
-
-        # content of $HOME/.pip/pip.conf
-        [global]
-        index-url = %(indexurl)s
-
-    and/or easy_install and some commands like "setup develop"::
-
-        # content of $HOME/.pydistutils.cfg
-        [easy_install]
-        index_url = %(indexurl)s
-
-
     %(cron)s
-    As a bonus, we have prepared an nginx config at:
+    We prepared an nginx configuration at:
 
         %(nginxconf)s
 
     which you might modify and copy to your /etc/nginx/sites-enabled
     directory.
     """) % locals())
-    tw.line("may quick pypi installations be with you :)", bold=True)
+    tw.line("may quick reliable pypi installations be with you :)",
+            green=True)
 
 
 def create_crontab(tw, etc, devpictl):
@@ -94,7 +108,7 @@ def create_crontab(tw, etc, devpictl):
     """ % (based, crontabpath))
 
 
-def create_devpictl(tw, tmpdir, httpport):
+def create_devpictl(tw, tmpdir):
     devpiserver = tmpdir.join("bin", "devpi-server")
     if not devpiserver.check():
         tw.line("created fake devpictl", red=True)
@@ -104,8 +118,7 @@ def create_devpictl(tw, tmpdir, httpport):
     devpictlpy = py.path.local(__file__).dirpath("ctl.py").read()
     devpictl = render(tw, devpiserver.dirpath(), "devpi-ctl",
                       firstline=firstline,
-                      httpport=httpport, devpictlpy=devpictlpy)
-    tw.line("wrote %s" % devpictl, bold=True)
+                      devpictlpy=devpictlpy)
     s = py.std.stat
     setmode = s.S_IXUSR  # | s.S_IXGRP | s.S_IXOTH
     devpictl.chmod(devpictl.stat().mode | setmode)
@@ -126,18 +139,21 @@ def gendeploy(config):
         tw.line("detected existing devpi-ctl, ensuring it is shut down",
                 red=True)
         subproc(tw, [devpi_ctl, "shutdown", "all"])
-        prefix = "re-"
-    tw.line("%sinstalling virtualenv to %s" % (prefix, target), bold=True)
-    try:
-        del os.environ["PYTHONDONTWRITEBYTECODE"]
-    except KeyError:
-        pass
-    subproc(tw, ["virtualenv", str(target)])
+    if not target.join("bin").check():
+        tw.line("creating virtualenv: %s" % (target, ), bold=True)
+        try:
+            del os.environ["PYTHONDONTWRITEBYTECODE"]
+        except KeyError:
+            pass
+        subproc(tw, ["virtualenv", "-q", str(target)])
+    else:
+        tw.line("using existing virtualenv: %s" %(target,), bold=True)
     pip = py.path.local.sysfind("pip", paths=[target.join("bin")])
-    tw.line("installing devpi-server and supervisor", bold=True)
+    tw.line("installing devpi-server,supervisor,eventlet into virtualenv",
+            bold=True)
     version = devpi_server.__version__
-    subproc(tw, [pip, "install", "devpi-server>=%s" % version, "supervisor"])
-    tw.line("generating configuration")
+    subproc(tw, [pip, "install", "--pre", "-q",
+                 "supervisor", "eventlet", "devpi-server>=%s" % version])
     gendeploycfg(config, target, tw=tw)
 
 def subproc(tw, args):
