@@ -147,12 +147,12 @@ class PTypedKey:
             current = next
         if position == -1:
             return []
-        l = []
+        names = set()
         for x in current:
             key = x.relto(self.keyfs.basedir)
             parts = key.split(os.sep)
-            l.append(parts[position])
-        return set(l)
+            names.add(parts[position])
+        return names
 
     def __repr__(self):
         return "<PTypedKey %r type %r>" %(self.key, self.type.__name__)
@@ -168,6 +168,12 @@ class DirKey:
         self.relpath = relpath
         self.filepath = self.keyfs._getpath(self.relpath)
 
+    def exists(self):
+        return self.filepath.check()
+
+    def delete(self):
+        return self.filepath.remove()
+
 class TypedKey:
     def __init__(self, keyfs, relpath, type):
         self.keyfs = keyfs
