@@ -14,23 +14,47 @@ from devpi_server.types import cached_property
 from devpi_server.config import parseoptions, configure_logging
 import devpi_server
 
-USE_FRONT = 1  # use front.pypi.org for getting simple pages
-
-
 def main(argv=None):
     """ devpi-server command line entry point. """
+    if argv is None:
+        argv = sys.argv
+    argv = map(str, argv)
+
     config = parseoptions(argv)
-    if config.args.version:
+    args = config.args
+
+    if args.version:
         print (devpi_server.__version__)
         return
 
-
-    if config.args.gendeploy:
+    if args.gendeploy:
         from devpi_server.gendeploy import gendeploy
         return gendeploy(config)
 
+    if args.start or args.stop or args.log or args.status:
+        xprocdir = config.serverdir.join(".xproc")
+        from devpi_server.bgserver import BackgroundServer
+        tw = py.io.TerminalWriter()
+        bgserver = BackgroundServer(tw, xprocdir)
+        if args.start:
+            return bgserver.start(args)
+        elif args.stop:
+            return bgserver.stop()
+        elif args.log:
+            return bgserver.log()
+        elif args.status:
+            if bgserver.info.isrunning():
+                bgserver.line("server is running with pid %s" %
+                              bgserver.info.pid)
+            else:
+                bgserver.line("no server is running")
+            return
+
     configure_logging(config)
     xom = XOM(config)
+    if args.passwd:
+        from devpi_server.db import run_passwd
+        return run_passwd(xom.db, config.args.passwd)
     return bottle_run(xom)
 
 def make_application():
@@ -44,19 +68,33 @@ def bottle_run(xom):
                          catchall=not xom.config.args.debug)
     port = xom.config.args.port
     log.info("devpi-server version: %s", devpi_server.__version__)
-    log.info("serving at url: http://%s:%s/",
-             xom.config.args.host, xom.config.args.port)
+    log.info("serverdir: %s" % xom.config.serverdir)
+    hostaddr = "http://%s:%s" %(xom.config.args.host, xom.config.args.port)
+    log.info("serving at url: %s", hostaddr)
     log.info("bug tracker: https://bitbucket.org/hpk42/devpi/issues")
     log.info("IRC: #pylib on irc.freenode.net")
     if "WEBTRACE" in os.environ and xom.config.args.debug:
         from weberror.evalexception import make_eval_exception
         app = make_eval_exception(app, {})
-    ret = bottle.run(app=app, server=xom.config.args.bottleserver,
+    bottleserver = get_bottle_server(xom.config.args.bottleserver)
+    log.info("bottleserver type: %s" % bottleserver)
+    ret = bottle.run(app=app, server=bottleserver,
                   host=xom.config.args.host,
                   reloader=False, port=port)
     xom.shutdown()
     return ret
 
+def get_bottle_server(opt):
+    if opt == "auto":
+        try:
+            import eventlet
+        except ImportError:
+            log.debug("could not import 'eventlet'")
+            opt = "wsgiref"
+        else:
+            opt = "eventlet"
+    return opt
+
 def add_keys(keyfs):
     # users and index configuration
     keyfs.USER = keyfs.addkey("{user}/.config", dict)
@@ -76,6 +114,7 @@ def add_keys(keyfs):
     keyfs.RELDESCRIPTION = keyfs.addkey(
             "{user}/{index}/{name}/{version}/description_html", bytes)
     keyfs.PATHENTRY = keyfs.addkey("{relpath}-meta", dict)
+    keyfs.ATTACHMENT = keyfs.addkey("+attach/{md5}/{type}/{num}", bytes)
     keyfs.FILEPATH = keyfs.addkey("{relpath}", bytes)
 
 class XOM:
@@ -122,10 +161,6 @@ class XOM:
         thread.start()
         return thread
 
-    @cached_property
-    def datadir(self):
-        return py.path.local(os.path.expanduser(self.config.args.datadir))
-
     @cached_property
     def releasefilestore(self):
         from devpi_server.filestore import ReleaseFileStore
@@ -134,7 +169,7 @@ class XOM:
     @cached_property
     def keyfs(self):
         from devpi_server.keyfs import KeyFS
-        keyfs = KeyFS(self.datadir)
+        keyfs = KeyFS(self.config.serverdir)
         add_keys(keyfs)
         return keyfs
 
@@ -158,7 +193,9 @@ class XOM:
     def httpget(self, url, allow_redirects, timeout=30):
         from requests.exceptions import RequestException
         headers = {}
+        USE_FRONT = self.config.args.bypass_cdn
         if USE_FRONT:
+            log.debug("bypassing pypi CDN for: %s", url)
             if url.startswith("https://pypi.python.org/simple/"):
                 url = url.replace("https://pypi", "https://front")
                 headers["HOST"] = "pypi.python.org"
@@ -233,8 +270,7 @@ class FatalResponse:
 
 def set_default_indexes(db):
     PYPI = "root/pypi"
-    DEV = "root/dev"
     if "root" not in db.user_list():
-        db.user_setpassword("root", "")
-    db.user_indexconfig_set(PYPI, bases=(), type="mirror", volatile=False)
-    db.user_indexconfig_set(DEV, bases=(PYPI,), type="stage", volatile=True)
+        db.user_create("root", password="")
+    if not db.index_exists(PYPI):
+        db.index_create(PYPI, bases=(), type="mirror", volatile=False)
