@@ -11,7 +11,6 @@ def test_create_zipfile(tmpdir):
     content = create_zipfile({"one": {"nested": "1"}, "two": {}})
     tmpdir.join("hello.zip").write(content, "wb")
 
-
 class TestStage:
     @pytest.fixture
     def stage(self, request, db):
@@ -19,29 +18,70 @@ class TestStage:
                       type="stage", volatile=True)
         if "bases" in request.fixturenames:
             config["bases"] = request.getfuncargvalue("bases")
-        db.user_indexconfig_set(**config)
+        db.index_create(**config)
         return db.getstage(user=config["user"], index=config["index"])
 
     def test_create_and_delete(self, db):
-        db.user_indexconfig_set(user="hello", index="world", bases=(),
-                                type="stage", volatile=False)
-        db.user_indexconfig_set(user="hello", index="world2", bases=(),
-                                type="stage", volatile=False)
-        db.user_indexconfig_delete(user="hello", index="world2")
-        assert not db.user_indexconfig_get(user="hello", index="world2")
-        assert db.user_indexconfig_get(user="hello", index="world")
+        db.index_create(user="hello", index="world", bases=(),
+                        type="stage", volatile=False)
+        db.index_create(user="hello", index="world2", bases=(),
+                        type="stage", volatile=False)
+        db.index_delete(user="hello", index="world2")
+        assert not db.index_get(user="hello", index="world2")
+        assert db.index_get(user="hello", index="world")
+
+    def test_set_and_get_acl(self, db):
+        db.index_create(user="hello", index="world", bases=(),
+                        type="stage", volatile=False)
+        indexconfig = db.index_get(user="hello", index="world")
+        # check that "hello" was included in acl_upload by default
+        assert indexconfig["acl_upload"] == ["hello"]
+        stage = db.getstage("hello/world")
+        # root cannot upload
+        assert not stage.can_upload("root")
+
+        # and we remove 'hello' from acl_upload ...
+        assert db.index_modify(user="hello", index="world", acl_upload=[])
+        # ... it cannot upload either
+        stage = db.getstage("hello/world")
+        assert not stage.can_upload("hello")
+
+    def test_getstage_normalized(self, db):
+        assert db.getstage("/root/pypi/").name == "root/pypi"
 
     def test_not_configured_index(self, db):
         stagename = "hello/world"
-        assert not db.getindexconfig(stagename)
+        assert not db.index_get(stagename)
         assert not db.getstage(stagename)
 
+    def test_indexconfig_set_throws_on_unknown_base_index(self, db):
+        with pytest.raises(db.InvalidIndexconfig) as excinfo:
+            db.index_create(user="hello", index="world",
+                            bases=("root/notexists", "root/notexists2"),)
+        messages = excinfo.value.messages
+        assert len(messages) == 2
+        assert "root/notexists" in messages[0]
+        assert "root/notexists2" in messages[1]
+
+    def test_indexconfig_set_throws_on_invalid_base_index(self, db):
+        with pytest.raises(db.InvalidIndexconfig) as excinfo:
+            db.index_create(user="hello", index="world",
+                            bases=("root/dev/123",),)
+        messages = excinfo.value.messages
+        assert len(messages) == 1
+        assert "root/dev/123" in messages[0]
+
+    def test_indexconfig_set_normalizes_bases(self, db):
+        ixconfig = db.index_create(user="hello", index="world",
+                                   bases=("/root/pypi/",))
+        assert ixconfig["bases"] == ("root/pypi",)
+
     def test_empty(self, stage, bases):
         assert not stage.getreleaselinks("someproject")
         assert not stage.getprojectnames()
 
-    def test_inheritance_simple(self, httpget, stage):
-        stage.configure(bases=("root/pypi",))
+    def test_inheritance_simple(self, httpget, stage, db):
+        stage._reconfigure(bases=("root/pypi",))
         httpget.setextsimple("someproject",
             "<a href='someproject-1.0.zip' /a>")
         entries = stage.getreleaselinks("someproject")
@@ -50,8 +90,57 @@ class TestStage:
         stage.register_metadata(dict(name="someproject", version="1.1"))
         assert stage.getprojectnames() == ["someproject",]
 
+    def test_inheritance_twice(self, httpget, db, stage):
+        db.index_create(user="root", index="dev2", bases=("root/pypi",))
+        stage_dev2 = db.getstage("root/dev2")
+        stage._reconfigure(bases=("root/dev2",))
+        httpget.setextsimple("someproject",
+            "<a href='someproject-1.0.zip' /a>")
+        stage_dev2.store_releasefile("someproject-1.1.tar.gz", "123")
+        stage.store_releasefile("someproject-1.2.tar.gz", "456")
+        entries = stage.getreleaselinks("someproject")
+        assert len(entries) == 3
+        assert entries[0].basename == "someproject-1.2.tar.gz"
+        assert entries[1].basename == "someproject-1.1.tar.gz"
+        assert entries[2].basename == "someproject-1.0.zip"
+        assert stage.getprojectnames() == ["someproject",]
+
+    def test_inheritance_normalize_multipackage(self, httpget, db, stage):
+        stage._reconfigure(bases=("root/pypi",))
+        httpget.setextsimple("some-project", """
+            <a href='some_project-1.0.zip' /a>
+            <a href='some_project-1.0.tar.gz' /a>
+        """)
+        stage.store_releasefile("some_project-1.2.tar.gz", "456")
+        entries = stage.getreleaselinks("some-project")
+        assert len(entries) == 3
+        assert entries[0].basename == "some_project-1.2.tar.gz"
+        assert entries[1].basename == "some_project-1.0.zip"
+        assert entries[2].basename == "some_project-1.0.tar.gz"
+        assert stage.getprojectnames() == ["some-project",]
+
+    def test_getreleaselinks_inheritance_shadow(self, httpget, stage):
+        stage._reconfigure(bases=("root/pypi",))
+        httpget.setextsimple("someproject",
+            "<a href='someproject-1.0.zip' /a>")
+        stage.store_releasefile("someproject-1.0.zip", "123")
+        entries = stage.getreleaselinks("someproject")
+        assert len(entries) == 1
+        assert entries[0].relpath.endswith("someproject-1.0.zip")
+
+    def test_getreleaselinks_inheritance_shadow_egg(self, httpget, stage):
+        stage._reconfigure(bases=("root/pypi",))
+        httpget.setextsimple("py",
+        """<a href="http://bb.org/download/py.zip#egg=py-dev" />""")
+        stage.store_releasefile("py-1.0.tar.gz", "123")
+        entries = stage.getreleaselinks("py")
+        assert len(entries) == 2
+        e0, e1 = entries
+        assert e0.basename == "py.zip"
+        assert e1.basename == "py-1.0.tar.gz"
+
     def test_inheritance_error(self, httpget, stage):
-        stage.configure(bases=("root/pypi",))
+        stage._reconfigure(bases=("root/pypi",))
         httpget.setextsimple("someproject", status_code = -1)
         entries = stage.getreleaselinks("someproject")
         assert entries == -1
@@ -59,7 +148,7 @@ class TestStage:
         #assert entries == -1
 
     def test_get_projectconfig_inherited(self, httpget, stage):
-        stage.configure(bases=("root/pypi",))
+        stage._reconfigure(bases=("root/pypi",))
         httpget.setextsimple("someproject",
             "<a href='someproject-1.0.zip' /a>")
         projectconfig = stage.get_projectconfig("someproject")
@@ -76,6 +165,18 @@ class TestStage:
         pconfig = stage.get_projectconfig("some")
         assert pconfig["1.0"]["+files"]["some-1.0.zip"].endswith("some-1.0.zip")
 
+    def test_project_config_shadowed(self, httpget, stage):
+        stage._reconfigure(bases=("root/pypi",))
+        httpget.setextsimple("someproject",
+            "<a href='someproject-1.0.zip' /a>")
+        content = "123"
+        entry = stage.store_releasefile("someproject-1.0.zip", content)
+        projectconfig = stage.get_projectconfig("someproject")
+        files = projectconfig["1.0"]["+files"]
+        link = files.values()[0]
+        assert link.endswith("someproject-1.0.zip")
+        assert projectconfig["1.0"]["+shadowing"]
+
     def test_store_and_delete_project(self, stage, bases):
         content = "123"
         entry = stage.store_releasefile("some-1.0.zip", content)
@@ -125,7 +226,7 @@ class TestStage:
         assert not filepath.join("_templ").check()
 
     def test_store_and_get_volatile(self, stage):
-        stage.configure(volatile=False)
+        stage._reconfigure(volatile=False)
         content = "123"
         content2 = "1234"
         entry = stage.store_releasefile("some-1.0.zip", content)
@@ -136,7 +237,7 @@ class TestStage:
         assert entry == 409
 
         # rewrite succeeds with volatile
-        stage.configure(volatile=True)
+        stage._reconfigure(volatile=True)
         entry = stage.store_releasefile("some-1.0.zip", content2)
         entries = stage.getreleaselinks("some")
         assert len(entries) == 1
@@ -148,6 +249,9 @@ class TestStage:
         stage.register_metadata(dict(name="hello", version="1.0", author="xy"))
         d = stage.get_metadata("hello", "1.0")
         assert d["author"] == "xy"
+        #stage.ixconfig["volatile"] = False
+        #with pytest.raises(stage.MetadataExists):
+        #    stage.register_metadata(dict(name="hello", version="1.0"))
 
     def test_releasedata_description(self, stage):
         source = py.std.textwrap.dedent("""\
@@ -183,34 +287,41 @@ class TestUsers:
         assert not db.user_validate("user", "password2")
 
     def test_create_and_delete(self, db):
-        db.user_setpassword("user", "password")
+        db.user_create("user", password="password")
+        assert db.user_exists("user")
         db.user_delete("user")
         assert not db.user_exists("user")
         assert not db.user_validate("user", "password")
 
     def test_create_and_list(self, db):
         baselist = db.user_list()
-        db.user_setpassword("user1", "password")
-        db.user_setpassword("user2", "password")
-        db.user_setpassword("user3", "password")
+        db.user_modify("user1", password="password")
+        db.user_modify("user2", password="password")
+        db.user_modify("user3", password="password")
         newusers = db.user_list().difference(baselist)
         assert newusers == set("user1 user2 user3".split())
         db.user_delete("user3")
         newusers = db.user_list().difference(baselist)
         assert newusers == set("user1 user2".split())
 
+    def test_server_passwd(self, db, monkeypatch):
+        from devpi_server.db import run_passwd
+        monkeypatch.setattr(py.std.getpass, "getpass", lambda x: "123")
+        run_passwd(db, "root")
+        assert db.user_validate("root", "123")
+
+    def test_server_email(self, db):
+        email_address = "root_" + str(id) + "@mydomain"
+        db.user_modify('root', email=email_address)
+        assert db.user_get("root")["email"] == email_address
+
+
 def test_setdefault_indexes(db):
     from devpi_server.main import set_default_indexes
     set_default_indexes(db)
-    ixconfig = db.getindexconfig("root/pypi")
+    ixconfig = db.index_get("root/pypi")
     assert ixconfig["type"] == "mirror"
 
-    ixconfig = db.getindexconfig("root/dev")
-    assert ixconfig["type"] == "stage"
-    assert ixconfig["bases"] == ("root/pypi",)
-    assert ixconfig["volatile"]
-
-
 def create_zipfile(contentdict):
     f = py.io.BytesIO()
     zip = py.std.zipfile.ZipFile(f, "w")
