@@ -31,6 +31,14 @@ class TestIndexParsing:
         """)
         assert len(result.releaselinks) == 3
 
+    def test_parse_index_normalized_name(self):
+        simplepy = DistURL("http://pypi.python.org/simple/ndg-httpsclient/")
+        result = parse_index(simplepy, """
+               <a href="../../pkg/ndg_httpsclient-1.0.tar.gz" />
+        """)
+        assert len(result.releaselinks) == 1
+        assert result.releaselinks[0].url.endswith("ndg_httpsclient-1.0.tar.gz")
+
     def test_parse_index_two_eggs_same_url(self):
         simplepy = DistURL("http://pypi.python.org/simple/Py/")
         result = parse_index(simplepy,
@@ -71,6 +79,14 @@ class TestIndexParsing:
         assert link.eggfragment == "py-dev"
         assert link2.basename == "py-1.0.zip"
 
+    def test_parse_index_with_wheel(self):
+        result = parse_index(self.simplepy,
+            """<a href="pkg/py-1.0-cp27-none-linux_x86_64.whl" />
+        """)
+        assert len(result.releaselinks) == 1
+        link, = result.releaselinks
+        assert link.basename == "py-1.0-cp27-none-linux_x86_64.whl"
+
     def test_parse_index_ftp_ignored_for_now(self):
         result = parse_index(self.simplepy,
             """<a href="http://bb.org/download/py-1.0.zip" />
@@ -103,6 +119,12 @@ class TestIndexParsing:
             <a href="http://bb.org/download/py-1.0.zip" />""")
         assert len(result.releaselinks) == 1
 
+    def test_parse_index_with_non_parseable_hrefs(self):
+        result = parse_index(self.simplepy,
+            """<a href="qlkwje 1lk23j123123" />
+            <a href="http://bb.org/download/py-1.0.zip" />""")
+        assert len(result.releaselinks) == 1
+
     def test_releasefile_and_scrape(self):
         result = parse_index(self.simplepy,
             """<a href="../../pkg/py-1.4.12.zip#md5=12ab">qwe</a>
@@ -112,7 +134,7 @@ class TestIndexParsing:
         assert len(result.releaselinks) == 1
         assert len(result.crawllinks) == 2
         result.parse_index(DistURL("http://pylib.org"), """
-               <a href="http://pylib.org/py-1.1.egg" />
+               <a href="http://pylib.org/py-1.1-py27.egg" />
                <a href="http://pylib.org/other" rel="download" />
         """, scrape=False)
         assert len(result.crawllinks) == 2
@@ -120,7 +142,7 @@ class TestIndexParsing:
         links = list(result.releaselinks)
         assert links[0].url == \
                 "http://pypi.python.org/pkg/py-1.4.12.zip#md5=12ab"
-        assert links[1].url == "http://pylib.org/py-1.1.egg"
+        assert links[1].url == "http://pylib.org/py-1.1-py27.egg"
 
     def test_releasefile_and_scrape_no_ftp(self):
         result = parse_index(self.simplepy,
@@ -168,6 +190,27 @@ class TestExtPYPIDB:
         assert link.md5 == "123"
         assert link.relpath.endswith("/pytest-1.0.zip")
 
+    def test_parse_project_replaced_eggfragment(self, extdb):
+        extdb.setextsimple("pytest", pypiserial=10, text='''
+            <a href="../../pkg/pytest-1.0.zip#egg=pytest-dev1" />''',)
+        links = extdb.getreleaselinks("pytest", refresh=10)
+        assert links[0].eggfragment == "pytest-dev1"
+        extdb.setextsimple("pytest", pypiserial=11, text='''
+            <a href="../../pkg/pytest-1.0.zip#egg=pytest-dev2" />''')
+        links = extdb.getreleaselinks("pytest", refresh=11)
+        assert links[0].eggfragment == "pytest-dev2"
+
+    def test_parse_project_replaced_md5(self, extdb):
+        extdb.setextsimple("pytest", pypiserial=10, text='''
+            <a href="../../pkg/pytest-1.0.zip#md5=123" />''',)
+        links = extdb.getreleaselinks("pytest", refresh=10)
+        assert links[0].md5 == "123"
+        extdb.setextsimple("pytest", pypiserial=11, text='''
+            <a href="../../pkg/pytest-1.0.zip#md5=456" />''')
+        links = extdb.getreleaselinks("pytest", refresh=11)
+        assert links[0].md5 == "456"
+
+
     def test_getprojectconfig(self, extdb):
         extdb.setextsimple("pytest", text='''
             <a href="../../pkg/pytest-1.0.zip#md5=123" />''')
@@ -309,16 +352,39 @@ def raise_ValueError():
 class TestRefreshManager:
 
     def test_pypichanges_nochanges(self, extdb, keyfs):
+        proxy = mock.create_autospec(XMLProxy)
+        proxy.list_packages_with_serial.return_value = {"hello": 10,
+                                                        "abc": 42}
+        proxy.changelog_since_serial.return_value = []
+        with pytest.raises(ValueError):
+            extdb.spawned_pypichanges(proxy, proxysleep=raise_ValueError)
+        proxy.list_packages_with_serial.assert_called_once_with()
+        assert keyfs.PYPISERIALS.get()["hello"] == 10
+        assert keyfs.PYPISERIALS.get()["abc"] == 42
+        assert extdb.getprojectnames() == ["abc", "hello"]
+
+    def test_pypichanges_mirrorversionchange(self, extdb, keyfs, monkeypatch):
         proxy = mock.create_autospec(XMLProxy)
         proxy.list_packages_with_serial.return_value = {"hello": 10}
         proxy.changelog_since_serial.return_value = []
         with pytest.raises(ValueError):
             extdb.spawned_pypichanges(proxy, proxysleep=raise_ValueError)
         proxy.list_packages_with_serial.assert_called_once_with()
-        assert keyfs.PYPISERIALS.get() == {"hello": 10}
+
+        # trying again should not trigger restart
+        proxy.list_packages_with_serial.called = False
+        with pytest.raises(ValueError):
+            extdb.spawned_pypichanges(proxy, proxysleep=raise_ValueError)
+        assert not proxy.list_packages_with_serial.called
+        monkeypatch.setattr(extdb, "MIRRORVERSION", extdb.MIRRORVERSION + 1)
+        # trying again should trigger restart
+        with pytest.raises(ValueError):
+            extdb.spawned_pypichanges(proxy, proxysleep=raise_ValueError)
+        assert proxy.list_packages_with_serial.called
+
 
     def test_pypichanges_changes(self, extdb, httpget, keyfs, monkeypatch):
-        keyfs.PYPISERIALS.set({"pytest": 20})
+        keyfs.PYPISERIALS.set({"pytest": 20, ".ver": extdb.MIRRORVERSION})
         httpget.setextsimple("pytest", '<a href="pytest-2.3.tgz"/a>',
                              pypiserial=20)
         assert len(extdb.getreleaselinks("pytest")) == 1
@@ -350,7 +416,7 @@ class TestRefreshManager:
                     keyfs, raise_error, monkeypatch, caplog):
         from xmlrpclib import ServerProxy
         got = []
-        keyfs.PYPISERIALS.set({"pytest": 10})
+        keyfs.PYPISERIALS.set({"pytest": 10, ".ver": extdb.MIRRORVERSION})
         def raise_xmlrpcish(since_int):
             got.append(since_int)
             raise_error()
