@@ -218,12 +218,14 @@ class TestKey:
 
     def test_dirkey(self, keyfs):
         key1 = keyfs.addkey("somedir", "DIR")
-        assert not hasattr(key1, "get")
-        assert not hasattr(key1, "set")
-        assert not hasattr(key1, "delete")
-        assert not hasattr(key1, "exists")
-
         assert key1.filepath.strpath.endswith("somedir")
         assert key1.filepath.strpath.endswith(key1.relpath)
+        assert not hasattr(key1, "get")
+        assert not hasattr(key1, "set")
+        assert not key1.exists()
+        key1.filepath.ensure("hello")
+        assert key1.exists()
+        key1.delete()
+        assert not key1.exists()
 
 
